/**************************************************************************************

  Interfacing PIC16F887 microcontroller with DHT22 sensor.
  Humidity (in rH%) and temperature (in C) are displayed on 16x2 LCD.
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 8MHz
  Configuration words: CONFIG1 = 0x2CD4
                       CONFIG2 = 0x0700
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

***************************************************************************************/

// LCD module connections
sbit LCD_RS at RD0_bit;
sbit LCD_EN at RD1_bit;
sbit LCD_D4 at RD2_bit;
sbit LCD_D5 at RD3_bit;
sbit LCD_D6 at RD4_bit;
sbit LCD_D7 at RD5_bit;
sbit LCD_RS_Direction at TRISD0_bit;
sbit LCD_EN_Direction at TRISD1_bit;
sbit LCD_D4_Direction at TRISD2_bit;
sbit LCD_D5_Direction at TRISD3_bit;
sbit LCD_D6_Direction at TRISD4_bit;
sbit LCD_D7_Direction at TRISD5_bit;
// end LCD module connections

// DHT22 pin connection (here data pin is connected to pin RB0)
#define DHT22_PIN         RB0_bit
#define DHT22_PIN_DIR     TRISB0_bit

#include <DHT22.c>  // include DHT22 driver source file

// main function
void main()
{
  OSCCON = 0x70;            // set internal oscillator to 8MHz
  ANSELH = 0;               // configure all PORTB pins as digital
  delay_ms(1000);           // wait a second

  Lcd_Init();               // initialize LCD module
  Lcd_Cmd(_LCD_CURSOR_OFF); // cursor off
  Lcd_Cmd(_LCD_CLEAR);      // clear LCD
  lcd_out(1, 1, "Temp =");
  lcd_out(2, 1, "Humi =");

  while(1)
  {
    int humidity, temperature;
    char temp_a[8], humi_a[8];
    
    // read humidity (in hundredths rH%) and temperature (in hundredths C) from the DHT22 sensor
    dht22_read(&humidity, &temperature);
    
    if(temperature < 0)
      sprinti(temp_a, "-%02u.%01u%cC", abs(temperature)/10, abs(temperature) % 10, 223);
    else
      sprinti(temp_a, " %02u.%01u%cC", temperature/10, temperature % 10, 223);
    
    if(humidity >= 1000)     // if humidity >= 100.0 rH%
      sprinti(humi_a, "%03u.%01u %%", humidity/10, humidity % 10);
    else
      sprinti(humi_a, " %02u.%01u %%", humidity/10, humidity % 10);
    
    lcd_out(1, 7, temp_a);
    lcd_out(2, 7, humi_a);

    //delay_ms(1000);  // wait a second

  }

}
// end of code.